@@ -1,7 +1,7 @@
 /* boot.c  -  Boot image composition */
 /*
 Copyright 1992-1997 Werner Almesberger.
-Copyright 1999-2004 John Coffman.
+Copyright 1999-2005 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -117,8 +117,7 @@ void boot_image(char *spec,IMAGE_DESCR *descr)
 #endif
 	    ) {
 		descr->flags |= FLAG_TOOBIG;
-		if (!nowarn)
-		fprintf(errstd, "Warning: The initial RAM disk is too big to fit between %s and\n"
+		warn("The initial RAM disk is too big to fit between %s and\n"
 				"   the 15M-16M memory hole."
 #ifndef LCF_INITRDLOW
 # if 0
@@ -131,7 +130,7 @@ void boot_image(char *spec,IMAGE_DESCR *descr)
 				"   be assumed that the BIOS supports memory moves above 16M."
 # endif
 #endif
-				"\n", hi_sectors ? "the kernel" : "1M");
+				, hi_sectors ? "the kernel" : "1M");
 	}
 	geo_close(&geo);
     }
@@ -385,17 +384,16 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	if (fstat(b_fd,&st) < 0)
 	    die("fstat %s: %s",boot,strerror(errno));
 	if (!geo.file) part_verify(st.st_rdev,0);
-	else if (st.st_size > SECTOR_SIZE && !nowarn) {
-	    fprintf(errstd,"Warning: 'other = %s' specifies a file that is longer\n"
+	else if (st.st_size > SECTOR_SIZE) {
+	    warn("'other = %s' specifies a file that is longer\n"
 	    	"    than a single sector.", boot);
 	    if (st.st_size >= SECTOR_SIZE*(SETUPSECS-1) &&
 	        read(b_fd, buff[0].sector, SECTOR_SIZE*(SETUPSECS-1)) == 
 	    		SECTOR_SIZE*(SETUPSECS-1)   &&
-	    	!strncmp(buff[2].sector+2,"HdrS",4)
+	    	!strncmp((char*)buff[2].sector+2,"HdrS",4)
 	    			) {
-		fprintf(errstd,"  This file may actually be an 'image ='");
+		warn("This file may actually be an 'image ='");
 	    }
-	    fprintf(errstd, "\n");
 	}
 	if (lseek(b_fd,(int) BOOT_SIG_OFFSET,SEEK_SET) < 0)
 	    die("lseek %s: %s",boot,strerror(errno));
@@ -420,7 +418,7 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	code = to_number(pos);
 	if (code >= 80 && code <= 89) {
 	    /* convert to 0x80 to 0x89 */
-	    fprintf(errstd,"Warning: Radix error, 'boot-as=%d' taken to mean 'boot-as=0x%x'\n",
+	    warn("Radix error, 'boot-as=%d' taken to mean 'boot-as=0x%x'",
 	    			code, code+0x30);
 	    code += 0x30;
 	}
