@@ -1010,23 +1010,25 @@ static int get_image(char *name,char *label,IMAGE_DESCR *descr)
 #ifdef LCF_VIRTUAL
     if ( (deflt = cfg_get_strg(cf_options,"vmdefault")) &&
 #ifdef LCF_IGNORECASE
-		!strcasecmp(deflt,label))
+		!strcasecmp(deflt,label))  {
 #else
-		!strcmp(deflt,label))
+		!strcmp(deflt,label))  {
 #endif
 	descrs.d.descr[this_image].flags |= FLAG_VMDEFAULT;
 	param2.flag2 |= FLAG2_VIRTUAL;
+	}
 #endif
 
 #ifdef LCF_NOKEYBOARD
     if ( (deflt = cfg_get_strg(cf_options,"nokbdefault")) &&
 #ifdef LCF_IGNORECASE
-		!strcasecmp(deflt,label))
+		!strcasecmp(deflt,label))  {
 #else
-		!strcmp(deflt,label))
+		!strcmp(deflt,label))  {
 #endif
 	descrs.d.descr[this_image].flags |= FLAG_NOKBDEFAULT;
 	param2.flag2 |= FLAG2_NOKBD;
+	}
 #endif
 
     return this_image;
