@@ -16,8 +16,10 @@ source directory.
 
 #ifndef LILO_ASM
 #include <fcntl.h>
-#include <sys/stat.h>
 #include <asm/types.h>
+#if !__MSDOS__
+#include <sys/stat.h>
+#endif
 
 #ifndef PAGE_SIZE
 # define PAGE_SIZE 4096U
@@ -35,6 +37,10 @@ source directory.
 
 #endif
 
+#if __MSDOS__
+#define const
+#endif
+
 
 /*
 ;*/typedef struct {		/*
@@ -366,8 +372,8 @@ extern LILO_EXTRA extra;
 extern char *identify;
 extern int verbose,test,compact,linear,nowarn,lba32,autoauto,passw,geometric;
 extern int bios_boot, bios_map;
-extern int ireloc, force_fs, extended_pt, query;
-extern int colormax;
+extern int ireloc, force_fs, force_raid, extended_pt, query;
+extern int colormax, warnings;
 extern DL_BIOS bios_passes_dl;
 extern int boot_dev_nr,raid_index,raid_flags,do_md_install,ndisk,zflag,eflag;
 extern unsigned short drv_map[DRVMAP_SIZE+1]; /* needed for fixup maps */
@@ -396,6 +402,10 @@ extern char *identify;	/* in identify.c */
 /* fprintf an error message and then exit. */
 
 
+/*volatile*/ void warn(char *fmt,...);
+/* issue a warning message if !nowarn */
+
+
 void *alloc(int size);
 /* Allocates the specified number of bytes. Dies on error. */
 
