@@ -176,6 +176,7 @@ static char buffer[SECTOR_SIZE];
 	      tsecs % 10);
 	}
 	printf("  %snattended booting\n", param2.flag2&FLAG2_UNATTENDED ? "U" : "No u");
+	printf("  %sPC/AT keyboard hardware prescence check\n", param2.flag2&FLAG2_NOKBD ? "" : "No ");
 	if (boot.par_1.prompt & FLAG_PROMPT) printf("  Always enter boot prompt\n");
 	else printf("  Enter boot prompt only on demand\n");
 	printf("  Boot-time BIOS data%s saved\n",
@@ -757,6 +758,12 @@ extern int has_partitions_beta(dev_t dev);	/* defined in geometry.c */
 }
 #endif
 		printf("\nCFLAGS = " CFLAGS "\n");
+#if defined(LCF_DEVMAPPER) && defined(HAS_LIBDEVMAPPER_H)
+                printf("With");
+#else
+                printf("Without");
+#endif
+                printf(" device-mapper\n");
 		printf("\nglibc version %d.%d\n", __GLIBC__, __GLIBC_MINOR__);
 		printf("Kernel Headers included from  %d.%d.%d\n",
 			LINUX_VERSION_CODE>>16,
