@@ -84,7 +84,7 @@ source directory.
 #endif
 
 /* definitions for pf_hard_disk_scan in device.c  */
-#if VERSION_MINOR>50 && DEBUG
+#ifdef DEBUG_PARTITIONS
 #define PARTITIONS "./devfs_partitions"
 #else
 #define PARTITIONS "/proc/partitions"
