@@ -8,7 +8,7 @@ source directory.
 
 */
 
-/*#define DEBUG*/
+/*#define DEBUG_PROBE*/
 #define BITMAP 0	/* change to 1 when do_bitmap is filled in */
 #define VOLID  1	/* change to 1 when do_volid is filled in */
 
@@ -283,7 +283,7 @@ static void do_help(void)
     			pr->str ? pr->str : "        ",
     			pr->help);
     }
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
     printf("    In some cases, the '-v' flag will produce more output.\n");
     printf("sizeof(video_t) = %d  sizeof(floppy_t) = %d  sizeof(hard_t) = %d\n"
            "sizeof(edd_t) = %d  sizeof(buf.s) = %d\n",
@@ -321,7 +321,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
 #else
     if (notice(4)) exit(0);
 #endif
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	printf("get_geom: drive = 0x%02X\n", drive);
 	fflush(stdout);
 #endif
@@ -336,7 +336,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
 	if (drive > buf.s5.disk) return 1;
 	if (!hdp[drive-0x80]) {
 	    i = get_geom(drive-1, geom);
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 		printf("get_geom recursive return = %d  AH=0x%02X\n", i, i-1);
 		fflush(stdout);
 #endif
@@ -344,7 +344,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
 	}
 	hd = hdp[drive-0x80];
     } else return 1;
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	printf("get_geom:  hd = %08X\n", (int)hd);
 	fflush(stdout);
 #endif
@@ -358,7 +358,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
     /* regs.eax = 0x1500;           check drive type */
     /* regs.edx = drive;			*/
 
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	printf("get_geom: int13, fn=15\n");
 	fflush(stdout);
 #endif
@@ -372,7 +372,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
    /* regs.eax = 0x0800;		*/
    /* regs.edx = drive;			*/
    
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	printf("get_geom: int13, fn=08\n");
 	fflush(stdout);
 #endif
@@ -413,14 +413,14 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
 #endif
    }
 
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
    printf("get_geom:  PT->%08X  S/N=%08X\n", (int)geom->pt, geom->serial_no);
 #endif
       
    /* regs.eax = 0x4100;      check EDD extensions present */
    /* regs.edx = drive;				*/
    /* regs.ebx = 0x55AA;			*/
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	printf("get_geom: int13, fn=41\n");
 	fflush(stdout);
 #endif
@@ -433,7 +433,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
       edd_t *dp;
 
       dp = (edd_t*)hdp[drive-0x80 + 1];
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	printf("get_geom:  EDD  dp = %08X\n", (int)dp);
 	fflush(stdout);
 #endif
@@ -443,7 +443,7 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
       /* regs.eax = 0x4800;		*/
       /* regs.edx = drive;		*/
       
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	printf("get_geom: int13, fn=48\n");
 	fflush(stdout);
 #endif
@@ -569,7 +569,7 @@ static void print_geom(int dr, struct disk_geom geom)
     char *sp = sz;
     unsigned int n, m=0;
 
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
     if (!dr) geom.n_total_blocks = 4000000000UL;	/* 2.09Tb */
 #endif
 
@@ -909,7 +909,7 @@ int bios_max_devs(void)
     return BIOS_MAX_DEVS;    
 }
 
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 static void dump_pt(unsigned char *pt)
 {
     int i, j;
@@ -944,7 +944,7 @@ int bios_device(GEOMETRY *geo, int device)
     if (fetch()) return -1;
     
     if (verbose>=5) printf("bios_dev:  device %04X\n", device);
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 	fflush(stdout);
 #endif    
     if (!has_partitions(device)) return -1;
@@ -982,7 +982,7 @@ int bios_device(GEOMETRY *geo, int device)
 	die("bios_device: read partition table");
     dev_close(&dev);
 
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
     if (verbose>=5) {
         printf("serial number = %08X\n", serial);
         dump_pt(part);
@@ -1000,7 +1000,7 @@ int bios_device(GEOMETRY *geo, int device)
 	if (verbose>=5) {
 		printf("bios_dev: (0x%02X)  vol-ID=%08X  *PT=%08lX\n",
 			bios, bdata.serial_no, (long)bdata.pt);
-#ifdef DEBUG
+#ifdef DEBUG_PROBE
 		dump_pt((void*)bdata.pt);
 #endif
 	}
